<?php

use League\Flysystem\HashPlugin;
use League\Flysystem\Filesystem;
use League\Flysystem\Adapter\Local;

class App {

	private $hooks;
	private $config;

	private static $utils;
	private static $instance;

	public function __construct() {


		self::$instance = $this;
		mb_internal_encoding("UTF-8");

		$this->setConfigs(); 
		$this->setCookies();
		$this->setConstants();
		$this->initClasses();
		$this->loadPlugins();
		
		isset($_GET['query']) && App::Search();
		isset($_GET['random']) && App::Random();
		isset($_GET['logout']) && User::Logout();

		$this->hooks->trigger('initial');

	}

	public static function get($var) {
		$self = self::$instance;
		return str2var($self->config, $var);
	}

	public static function set($key, $var) {
		$self = self::$instance;
		$self->config->$key = $var;
	}

	private function initClasses() {

		Factory::set('Database', new Database($this->config->sql));
		unset($this->config->sql); Factory::set('User', new User);
		
		self::$utils = Factory::set('Utils', new Utils);
		$this->hooks = Factory::set('Hooks', new Hooks);

	}

	private function setConfigs() {
		
		$settings = dirname(__DIR__) . '/Settings.php';
		
		if ( file_exists($settings) ) {
			$ini = INI::getConfig();
			$def = include($settings);
			$this->append($def, $ini);
		} else { $this->_installer(); }

	}

	private function append($def, $ini) {

		$folder  = $def['subfolder'];
		$rewrite = $ini["mod_rewrite"];

		$def['app_path'] = '/';
		empty($folder) ?: $def['app_url'] .= '/' . $folder;
		empty($folder) ?: $def['app_path'] .= $folder . '/';
		
		$this->config = (object) array_replace($def, $ini ?: []);

    }
    
    private function setFilesystem() {
        
        // Flysystem adapter for local drive
        $adapter = new Local(PATH_ROOT, LOCK_EX, Local::DISALLOW_LINKS, [
            'file' => ['public' => 0666, 'private' => 0664],
            'dir'  => ['public' => 0777, 'private' => 0775]
        ]);
        
        $Flysystem = new Filesystem(
            $adapter, ['visibility' => 'public']
        );
        
        $Flysystem->addPlugin(new HashPlugin);
        Factory::set('Flysystem', $Flysystem);

    }

	private function setConstants() {

		define('APP_VERSION', '2.00');
		define('PATH_ROOT', $this->RootPath($this->config->root));

		$date = date("My/Dd");
		$hash = hash('crc32b', session_id() . $_SERVER['REMOTE_ADDR']);
		$page = App::get('app_scheme') .'://'. App::get('app_host') . $_SERVER["SCRIPT_URL"];
      Cookie::set('hash', $hash);

		$constants = [

			'PATH_LIBS'  => PATH_ROOT . '/Libs',
			'PATH_UPLD'  => PATH_ROOT . '/Uploads',
			'PATH_SAVE'  => PATH_ROOT . '/Uploads/Media/' . $date,
			'PATH_TEMP'  => PATH_ROOT . '/Uploads/Temp/'  . $hash,
			'PATH_CLASS' => PATH_ROOT . '/Libs/Classes',
			'MEDIA'      => PATH_ROOT . '/Uploads/Media/',
			'THEME'      => PATH_ROOT . '/Template/Theme',
			'CLASSES'    => PATH_ROOT . '/Libs/Classes',
			'RESOURCES'  => PATH_ROOT . '/Libs/Classes/Resources',

			'DATE'       => $date, 'HASH' => $hash,
			'DOMAIN'     => $this->config->app_url,
			'SAVE_DIR'   => 'Uploads/Media/' . $date,
			'USER_TEMP'  => 'Uploads/Temp/' . $hash,
			'SR'         => $this->config->mod_rewrite == "slash" ? '/' : ',',
			'Language'   => PATH_ROOT . '/Libs/Lang/' . $this->config->app_lang.'.php',

			'PAGE_URL'   => $page,
			'UNIQ_ID'    => hash('md5', $page),
            
			// Paths for Flysystem
			'UP_CACHE'  => 'update.ini',
			'TEMP_DIR'  => 'Uploads/Temp/' . $hash,
			'MEDIA_DIR' => 'Uploads/Media'

		];

		$this->defineConstants($constants);

	}

	private function RootPath($path = '') {
		return empty($path)
			? dirname(dirname(__DIR__))
			: $this->ValidatePath($path);
	}

	private function ValidatePath($path) {
		$root = rtrim($path, '/');
		return is_dir("$root/Libs") ? $root : die('Provided root path is invalid.');
	}

	// Global, static access to utils methods
	public static function __callStatic($func, $args) {
		if (method_exists(self::$utils, $func)) { 
			return call_user_func_array([self::$utils, $func], $args); 
		}
	}

	public static function Search() {
		$key = filter_input(INPUT_GET, 'query', FILTER_SANITIZE_STRING);
		$query = self::addLine($key);
		header("Location: " . self::get('app_url') . '/search' . SR . $query, true, 301);
	}

	public static function Redirect($page = null) {
		if (isset($page)) { if ($page == 404) header("Location: page," . $page, true, 301); }
		else { header("Location: " . self::get('app_url') ); }
	}

	public static function Random() {
		$data = Factory::get('Database')->orderBy('rand()')->getOne('posts', ['id','tytul','type','category']);
		header("Location: " . self::postLink($data, true), true, 301);
	}

	private function loadPlugins() {
		$plugins = include PATH_LIBS . '/Hooks.php';
		foreach ($plugins as $action => $callbacks) {
			$this->hooks->attach($action, $callbacks);
		}
	}

	public static function Config($vars = null) {

		if (is_null($vars)) {
			return self::$instance->config;
		}
		
		$output = new stdClass;
		$conf = self::$instance->config;

		if (strpos($vars, ',')) {
			$params = array_map('trim', explode(',', $vars));
			foreach ($params as $param) {
				$output->{$param} = isset($conf->{$param})
					? $conf->{$param} : null;
			} return $output;
		}

		return isset($conf->$vars) ? $conf->$vars : null;

	}

	public static function getConfig($json = false) {
		$conf = clone self::$instance->config;
		$conf->logged = User::Logged();
		$rm = ['code','root','seed','hashseed','cryptseed','adm_key','sql'];
		foreach ($rm as $i => $val) { unset($conf->{$val}); }
		$conf->lang = preg_replace('/_(\w+)/','', $conf->app_lang);
		return $json ? json_encode($conf) : $conf;
	}

	private function defineConstants($array) {
		foreach ($array as $key => $value) {
			defined($key) ?: define($key, $value);
		}
	}

	private function setCookies() {
		Cookie::is('autoplay') ?: Cookie::set('autoplay', $this->config->autoplay);
		Cookie::is('pagemode') ?: Cookie::set('pagemode', $this->config->page_mode);
		Cookie::is('viewmode') ?: Cookie::set('viewmode', $this->config->page_view_mode);
	}

	private function _installer() {
		$prefix = dirname(dirname(__DIR__));
		if (stream_resolve_include_path($prefix . '/Install/index.php')) {
			$host = getScheme() .'://'. $_SERVER['HTTP_HOST'];
			$self = rtrim(dirname($_SERVER['PHP_SELF']), '/');
			header('Location: ' . $host . ($self ?: '') . '/Install');
		} else { echo 'Installation folder missing. Please reupload it and try again.'; }
		exit;
	}

} // END class