<?php

// Copyright (C) 2018  Avidi Media Script

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

class Hooks
{

	private static $_instance;
	private $_theHooks = array();
	private $_theEvents = array();

	public static function getInstance() // Singleton
	{
		if (self::$_instance === null || !(self::$_instance instanceof self)) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	// ::::::::::::::::: Multiple listeners :::::::::::::::::
	// Omitting regular functions and closures, attach method 
	// can also initialize classes and handle methods

	public function attach($eventName, $callback, $method = null)
	{
		$args = compact('eventName', 'callback', 'method');
		if ($method === null) { unset($args['method']); }

		if ($action = $this->_parseCallback($args)) {
			$this->_prepareEvent($eventName);
			$this->_theEvents[$eventName][] = $action;
		}
	}

	public function detach($eventName = null)
	{
		if ($this->_hasEvent($eventName)) {
			unset($this->_theEvents[$eventName]);
			return;
		}
		$this->_theEvents = array();
	}

	public function trigger($eventName, $args = null)
	{
		if ($this->_hasEvent($eventName)) {
			foreach ($this->_theEvents[$eventName] as $action) {
				$this->_call($action, $args);
			}
		}
	}

  // :::::::::::::::: Single listener per event ::::::::::::::::

	public function on($eventName, $callback, $method = null)
	{
		$compact = compact('eventName', 'callback', 'method');
		if ($action = $this->_parseCallback($compact)) {
			$this->_prepareHook($eventName);
			$this->_theHooks[$eventName] = $action;
		}
	}

	public function off($eventName = null)
	{
		if ($this->_hasHook($eventName)) {
			unset($this->_theHooks[$eventName]);
		}
	}

	public function fire($eventName, $args = null)
	{
		if ($this->_hasHook($eventName) === false) {
			return false;
		}

		$event = $this->_theHooks[$eventName];
		return $this->_call($event, $args, true);
	}

  // -----------------------------
  // ========== Helpers ==========
  // -----------------------------

	private function _call($params, $args = null, $output = false)
	{
		$args = is_array($args) ? $args : array();

		if ($params->type === "function") {

			$data = call_user_func_array($params->action, $args);

		} elseif ($params->type === "class") {

			$reflection = new ReflectionClass($params->action);
			$reflection->newInstanceArgs($args);

		} elseif ($params->type === "method") {

			$reflectionMethod = new ReflectionMethod($params->action, $params->method);
			$data = $reflectionMethod->invokeArgs($params->statis ? null : new $params->action, $args);

		}

		if ($output && isset($data)) return $data;
	}

	private function _hasEvent($eventName)
	{
		return array_key_exists($eventName, $this->_theEvents);
	}

	private function _hasHook($eventName)
	{
		return array_key_exists($eventName, $this->_theHooks);
	}

	private function _prepareEvent($eventName)
	{
		if (!isset($this->_theEvents[ $eventName ])) {
			$this->_theEvents[$eventName] = array();
		}
	}

	private function _prepareHook($eventName)
	{
		if (!isset($this->_theHooks[ $eventName ])) {
			$this->_theHooks[$eventName] = array();
		}
	}

	private function _parseCallback($args)
	{
		extract($args); // Exports $eventName, $callback and $method

		if (is_callable($callback) && (function_exists($callback) || $callback instanceof Closure)) {

			return (object) ['type' => 'function', 'action' => $callback];

		} elseif (is_string($callback) && class_exists($callback)) {

			if (count($args) === 2) { // No method defined, initializing class

				return (object) ['type' => 'class', 'action' => $callback];

			} elseif (count($args) === 3 && method_exists($callback, $method) && is_callable([$callback, $method])) {

				$reflect = new ReflectionMethod($callback, $method);
				return (object) [
					'type' => 'method', 
					'action' => $callback, 
					'method' => $method,
					'static' => $reflect->isStatic()
				];

			}

		} elseif (is_array($callback)) {

			// :::::::::::: Array of Events ::::::::::::

			foreach ($callback as $action) {
				if (is_array($action)) {
					array_unshift($action, $eventName);
					call_user_func_array([$this, 'attach'], $action);
				} else {
					$this->attach($eventName, $action);
				}
			}

		}

		return false;
	}

}