<?php

class Submit {
   
   use Prepare;
   
   protected $sql;
   protected $usr;
   protected $raw;
   protected $type;
   protected $post;
   protected $user;
   protected $notes;
   protected $output;

   public function __construct($type) {

      $this->submissionDisabled();
      User::can('submit.content') ?: App::Redirect();

      $this->note  = Factory::get('Notes');
      $this->sql   = Factory::get('Database');
      $this->web   = Factory::get('WebData');
      $this->usr   = Factory::get('User');
      $this->user  = $this->usr->getData();

      $this->type = $this->postType($type);
      $this->post = $this->preparePost();
      $this->prepareData();

      App::createTempdirDir();
      
   }
   
   private function submissionDisabled() {
      $can = App::get('disable_submission') && !User::isAdmin();
      $can && die( Lang::get('submission_disabled') );
   }
   
   protected function countTags($tags) {
      $tagsArray = array_map('trim', explode(',',$tags));
      if ( App::get('tags_limit') && (sizeof($tagsArray) > App::get('tags_limit')) ) {
         die(Lang::get('maxtags_error'));
      }
   }
   
   protected function addTags($tags) {
      $tagsArray = array_map('trim', explode(',',$tags));
      foreach ($tagsArray as $tag) { if (empty($tag)) continue;
         if (!$this->sql->countRows('tags', 'keyword=?', $tag)) {
            $this->sql->insert('tags', ['keyword' => $tag]);
         }
      }
   }

   private function isDraft() {
      return isset($this->post->saveas) && $this->post->saveas == "draft";
   }
   
   private function prepareData() {

      $tags = $this->prepareTags($this->post->tags);
      
      $this->output = (object) [
         'time' => time(),
         'type' => $this->type,
         'tags' => $tags->tags,
         'logged' => $this->user->logged,
         'status' => $this->getUserStatus(),
         'category' => $this->post->category,
         'title' => $this->isValid('title', $this->post->title),
         'desc' => $this->isValid('desc', $this->post->desc),
         'user' => $this->isValid('user', $this->user->username ?: $this->post->nick)
      ];
      
   }
   
   private function getUserStatus() {

      if ($this->isDraft()) { return -1; }
      
      /* -------------------------------------------

      -2 - Hidden     // Visibility: invisible
      -1 - Draft      // Visibility: invisible
       0 - Moderation // Visibility: invisible
       1 - Moderation // Visibility: profiles
       2 - Published  // Visibility: everywhere

      */
      
      $conf = App::Config('in_profile,autoaccept');
      
      if ($this->user->logged) {
         if ($this->user->isadmin || $conf->autoaccept) { return 2; }
         if ($conf->in_profile) { return 1; }
      }
      
      return 0;

   }
   
   private function isValid($field, $value) {

      $i = 0; $error = 0;
      $ruleset = $this->getRules( $field );
      
      if ($ruleset) {
         
         foreach ( $ruleset as $rule => $cond ) {
            
            switch ($rule) {
               case 'not_empty': if (empty($value)) { $error++; } break;
               case 'regex_test': if (!preg_match($cond[0], $value)) { $error++; } break;
               case 'min_length': if (strlen($value) < $cond[0]) { $error++; } break;
               case 'max_length': if (strlen($value) > $cond[0]) { $error++; } break;
               case 'callback': 
                  $fieldnc = (gettype($cond) === 'array') ? $cond[0] : $cond;
                  if (call_user_func($fieldnc, $value)) { $error++; } break;
            } 
            
            if ($error) { 
               $info = $cond[1];
               echo $this->Error($info) ? $this->Error($info) : $info; exit; 
            } 
            
            $i++; 

         } // END loop
         
      } // if $ruleset
      
      if (!$error) return $value;
      
   }
   
   private function getRules($field) {
      
      $E = new JSobject([
         'login_used' => function($value) {
            if ( !$this->user->logged && User::userExists($value) ) $this->Error('username_taken');
         }
      ]);
      
      $rules = [
         'user' => [
            'not_empty' => [ true, Lang::get('empty_username') ],
            'min_length' => [ App::get('user_minlen') , Lang::get('user_minlen') ],
            'max_length' => [ App::get('user_maxlen') , Lang::get('user_maxlen') ],
            'regex_test' => [ '/^[0-9A-Za-z_-]+$/i' , Lang::get('login_invalid') ],
            'callback' => $E->login_used
         ],
         'title' => [
            'not_empty' => [ true, Lang::get('empty_title') ],
            'min_length' => [ App::get('title_minlen') , Lang::get('title_minlen') ],
            'max_length' => [ App::get('title_maxlen') , Lang::get('title_maxlen') ]
         ]
      ];
      
      return $rules[ $field ];
     
   }
  
   protected function mkSaveDir($dir) {
     if (!is_dir($dir)) {
        $oldmask = umask(0);
        mkdir($dir, 0777, true);
        umask($oldmask);
      }
   }
   
   protected function makeFolder($postID) {
      
      $date = DATE . '/' . $postID;
      $path = SAVE_DIR . '/' . $postID;
      $save = PATH_SAVE .'/'. $postID;
      
      $this->mkSaveDir($save);
      
      return (object) ['save' => $save, 'date' => $date, 'path' => $path];

   }
   
   protected function moveFiles($destination) {
      foreach (glob(PATH_TEMP.'/*') as $file) {
         rename($file, $destination .'/'. basename($file));
      } return glob( $destination .'/*' );
   }
   
   protected function clearFolder() {
      if ( is_dir(PATH_TEMP) ) {
         $files = glob(PATH_TEMP . '/*');
         foreach( $files as $file ) {
            if ( is_file($file) ) { unlink($file); }
         } @rmdir(PATH_TEMP);
      }
   }
   
   private function postType($type) {
      $types = ['article', 'gif', 'photos', 'video','audio'];
      return in_array($type, $types) ? $type : exit;
   }
   
   protected function Error($e) {
      $error = [
         'no_images' => Lang::get('art_add_photo'), 
         'submit_error' => Lang::get('post_submit_error'),
         'title_invalid' => Lang::get('title_invalid_chars'),
         'img_invalid_url' => Lang::get('img_invalid_url'),
         'title_min_length' => Lang::get('title_minlen'),
         'login_used' => Lang::get('username_taken'),
         'login_invalid' => Lang::get('login_invalid'),
         'user_min_len' => Lang::get('user_minlen'),
         'user_max_len' => Lang::get('user_maxlen'),
         'desc_length' => Lang::get('art_too_short'),
      ]; if ($error[$e]) { echo $error[$e]; exit; } else { return false; }
   }
   
} // END Class